using Adaptation.Eaf.Management.ConfigurationData.CellAutomation;
using Adaptation.Ifx.Eaf.EquipmentConnector.File.Configuration;
using Adaptation.Shared;
using Adaptation.Shared.Duplicator;
using Adaptation.Shared.Methods;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Text.Json;

namespace Adaptation.FileHandlers.txt;

public class FileRead : Shared.FileRead, IFileRead
{

    private long? _TickOffset;
    private readonly string _OriginalDataBioRad;

    public FileRead(ISMTP smtp, Dictionary<string, string> fileParameter, string cellInstanceName, int? connectionCount, string cellInstanceConnectionName, FileConnectorConfiguration fileConnectorConfiguration, string equipmentTypeName, string parameterizedModelObjectDefinitionType, IList<ModelObjectParameterDefinition> modelObjectParameters, string equipmentDictionaryName, Dictionary<string, List<long>> dummyRuns, Dictionary<long, List<Shared.Metrology.WS.Results>> staticRuns, bool useCyclicalForDescription, bool isEAFHosted) :
        base(new Description(), false, smtp, fileParameter, cellInstanceName, connectionCount, cellInstanceConnectionName, fileConnectorConfiguration, equipmentTypeName, parameterizedModelObjectDefinitionType, modelObjectParameters, equipmentDictionaryName, dummyRuns, staticRuns, useCyclicalForDescription, isEAFHosted: connectionCount is null)
    {
        _MinFileLength = 10;
        _NullData = string.Empty;
        _Logistics = new(this);
        if (_FileParameter is null)
            throw new Exception(cellInstanceConnectionName);
        if (_ModelObjectParameterDefinitions is null)
            throw new Exception(cellInstanceConnectionName);
        if (_IsDuplicator)
            throw new Exception(cellInstanceConnectionName);
        _OriginalDataBioRad = "OriginalDataBioRad_";
        if (_IsEAFHosted)
            NestExistingFiles(_FileConnectorConfiguration);
    }

    void IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, Exception exception) => Move(extractResults);

    void IFileRead.WaitForThread() => WaitForThread(thread: null, threadExceptions: null);

    string IFileRead.GetEventDescription()
    {
        string result = _Description.GetEventDescription();
        return result;
    }

    List<string> IFileRead.GetHeaderNames()
    {
        List<string> results = _Description.GetHeaderNames();
        return results;
    }

    string[] IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, string to, string from, string resolvedFileLocation, Exception exception)
    {
        string[] results = Move(extractResults, to, from, resolvedFileLocation, exception);
        return results;
    }

    JsonProperty[] IFileRead.GetDefault()
    {
        JsonProperty[] results = _Description.GetDefault(this, _Logistics);
        return results;
    }

    Dictionary<string, string> IFileRead.GetDisplayNamesJsonElement()
    {
        Dictionary<string, string> results = _Description.GetDisplayNamesJsonElement(this);
        return results;
    }

    List<IDescription> IFileRead.GetDescriptions(IFileRead fileRead, List<Test> tests, IProcessData processData)
    {
        List<IDescription> results = _Description.GetDescriptions(fileRead, _Logistics, tests, processData);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.GetExtractResult(string reportFullPath, string eventName)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        if (string.IsNullOrEmpty(eventName))
            throw new Exception();
        _ReportFullPath = reportFullPath;
        DateTime dateTime = DateTime.Now;
        results = GetExtractResult(reportFullPath, dateTime);
        if (results.Item3 is null)
            results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(results.Item1, Array.Empty<Test>(), JsonSerializer.Deserialize<JsonElement[]>("[]"), results.Item4);
        if (results.Item3.Length > 0 && _IsEAFHosted)
            WritePDSF(this, results.Item3);
        UpdateLastTicksDuration(DateTime.Now.Ticks - dateTime.Ticks);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.ReExtract()
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        List<string> headerNames = _Description.GetHeaderNames();
        Dictionary<string, string> keyValuePairs = _Description.GetDisplayNamesJsonElement(this);
        results = ReExtract(this, headerNames, keyValuePairs);
        return results;
    }

#nullable enable

    private Tuple<string, Test[], JsonElement[], List<FileInfo>> GetExtractResult(string reportFullPath, DateTime dateTime)
    {
        if (dateTime == DateTime.MinValue)
            throw new ArgumentNullException(nameof(dateTime));
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results = new(string.Empty, Array.Empty<Test>(), Array.Empty<JsonElement>(), new List<FileInfo>());
        _TickOffset ??= 0; // new FileInfo(reportFullPath).LastWriteTime.Ticks - dateTime.Ticks;
        _Logistics = new Logistics(this, _TickOffset.Value, reportFullPath, useSplitForMID: true);
        SetFileParameterLotID(_Logistics.MID);
        FileInfo fileInfo = new(reportFullPath);
        if (fileInfo.Length < _MinFileLength)
            results.Item4.Add(fileInfo);
        else
        {
            try
            {
                ReadOnlyCollection<Run>? runs = Run.Get(_TickOffset.Value, _Logistics, results.Item4);
                // if (run is null)
                //     throw new Exception(string.Concat("A) No Data - ", dateTime.Ticks));
                if (runs.Count == 0)
                { }
            }
            catch (Exception)
            { }
            bool isBioRad;
            string dataText;
            string cassetteID;
            string fileNameTemp;
            string tupleFileName;
            DateTime cassetteDateTime;
            string directoryName = Path.GetDirectoryName(reportFullPath) ?? throw new Exception();
            string sequenceDirectoryName = Path.Combine(directoryName, _Logistics.Sequence.ToString());
            string originalDataBioRad = Path.Combine(directoryName, $"{_OriginalDataBioRad}{_Logistics.Sequence}.txt");
            IProcessData iProcessData = new ProcessData(this, _Logistics, _TickOffset.Value, results.Item4, _OriginalDataBioRad);
            if (_IsEAFHosted && iProcessData.Details.Count != 0)
            {
                if (!Directory.Exists(sequenceDirectoryName))
                    _ = Directory.CreateDirectory(sequenceDirectoryName);
                File.Move(reportFullPath, originalDataBioRad);
                _Log.Debug(string.Concat("****Extract() - Renamed [", reportFullPath, "] to [", originalDataBioRad, "]"));
            }
            for (int i = 0; i < iProcessData.Details.Count; i++)
            {
                if (iProcessData.Details[i] is not Tuple<string, bool, DateTime, string> tuple)
                    continue;
                isBioRad = tuple.Item2;
                dataText = tuple.Item4;
                cassetteID = tuple.Item1;
                cassetteDateTime = tuple.Item3;
                if (isBioRad)
                    tupleFileName = string.Concat("DetailDataBioRad_", cassetteID, "_", cassetteDateTime.Ticks, ".txt");
                else
                    tupleFileName = string.Concat("CassetteDataBioRad_", cassetteID, "_", cassetteDateTime.Ticks, ".txt");
                fileNameTemp = Path.Combine(sequenceDirectoryName, tupleFileName);
                if (!_IsEAFHosted)
                    continue;
                File.WriteAllText(fileNameTemp, dataText);
                File.SetLastWriteTime(fileNameTemp, cassetteDateTime);
                if (_Logistics.Sequence != cassetteDateTime.Ticks && File.Exists(originalDataBioRad))
                    File.Copy(originalDataBioRad, Path.Combine(directoryName, $"{_OriginalDataBioRad}{cassetteDateTime.Ticks}.txt"));
                File.Move(fileNameTemp, Path.Combine(directoryName, tupleFileName));
            }
            if (!_IsEAFHosted)
                results = iProcessData.GetResults(this, _Logistics, results.Item4);
            else
            {
                if (Directory.Exists(sequenceDirectoryName))
                    Directory.Delete(sequenceDirectoryName);
            }
        }
        return results;
    }

}